/***

IPTraf
An IP-oriented Network Statistics Utility for Ethernet and SLIP/PPP networks
   		
Written by Gerard Paul Java
Copyright (c) Gerard Paul Java 1997, 1998
   	
Version 1.2.0

This software is open source; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License in the included COPYING file for
details.

***/

#include <stdlib.h>
#include <unistd.h>
#include <curses.h>
#include <panel.h>
#include <signal.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "dirs.h"
#include "deskman.h"
#include "menurt.h"
#include "stdwinset.h"
#include "utfdefs.h"
#include "othfilter.h"
#include "utfilter.h"
#include "ifstats.h"
#include "serv.h"
#include "options.h"
#include "externs.h"
#include "log.h"
#include "attrs.h"
#include "error.h"

extern void srpromisc(int mode);
extern void about();

/*
 * Get the ball rolling: The program interface routine.
 */

void program_interface(struct OPTIONS *options, int argc,
		       int opt, char *optarg)
{
    struct MENU menu;
    int endloop = 0;
    int row = 1;
    int aborted;
    int faborted;
    int filtered = 0;
    struct filterlist fl;
    struct othpoptions ofilter;
    char ifname[10];
    struct porttab *ports;

    draw_desktop();
    attrset(STDATTR);
    move(0, 1);
    printw("IPTraf %s", VERSION);

    loadsavedtcpfilter(&fl, &filtered);
    loadothfilter(&ofilter);
    indicate("");
    loadaddports(&ports);

    if (argc == 1) {
	about();

	initmenu(&menu, 12, 41, 7, 18);

	additem(&menu, " IP traffic ^m^onitor", "Displays current IP traffic information");
	additem(&menu, " General interface ^s^tatistics", "Displays some statistics for attached interfaces");
	additem(&menu, " ^D^etailed interface statistics", "Displays more statistics for a selected interface");
	additem(&menu, " TCP/UDP ser^v^ice monitor", "Displays statistics on TCP/UDP traffic");
	additem(&menu, " ^E^thernet station monitor", "Displays statistics on detected Ethernet stations");
	additem(&menu, " ^T^CP display filters", "Manages TCP display filters");
	additem(&menu, " Other ^p^rotocol filters", "Select which non-TCP protocols to display");
	additem(&menu, " Ethernet ^h^ost descriptions", "Manages descriptions for Ethernet addresses");
	additem(&menu, " ^O^ptions", "Set various program options");
	additem(&menu, " E^x^it", "Exits program");

	endloop = 0;

	do {
	    showmenu(&menu);
	    operatemenu(&menu, &row, &aborted);

	    switch (row) {
	    case 1:
		ipmon(options->revlook, options->servnames, options->logging,
		      filtered, &fl, &ofilter, options->timeout);
		break;
	    case 2:
		ifstats(options->logging, options->logspan);
		break;
	    case 3:
		selectiface(ifname, &aborted);
		if (!aborted)
		    detstats(ifname, options->logging, options->logspan);
		break;
	    case 4:
		selectiface(ifname, &aborted);
		if (!aborted)
		    servmon(ifname, ports, options->logging, options->logspan,
			    options->servnames);
		break;
	    case 5:
		hostmon(options->logging, options->logspan);
		break;
	    case 6:
		tcpfilterselect(&fl, &filtered, &faborted);
		break;
	    case 7:
		othfilterselect(&ofilter);
		saveothfilter(&ofilter);
		break;
	    case 8:
		ethdescmgr();
		break;
	    case 9:
 		setoptions(options, &ports);
		saveoptions(options);
		break;
	    case 10:
		endloop = 1;
		break;
	    }
	} while (!endloop);

	destroymenu(&menu);
    } else {
	switch (opt) {
	case 'i':
  	    ipmon(options->revlook, options->servnames, options->logging,
	          filtered, &fl, &ofilter, options->timeout);
	    break;
	case 'g':
	    ifstats(options->logging, options->logspan);
	    break;
	case 'd':
	    detstats(optarg, options->logging, options->logspan);
	    break;
	case 's':
	    servmon(optarg, ports, options->logging, options->logspan,
		    options->servnames);
	    break;
	case 'e':
	    hostmon(options->logging, options->logspan);
	    break;
	}
    }

    destroyporttab(ports);
    erase();
    update_panels();
    doupdate();
}

/*
 * Handler for the TERM signal.  There's nothing we can do for the KILL.
 */

void termhandler()
{
    erase();
    refresh();
    endwin();
    unlink(IPTIDFILE);
    fprintf(stderr, "Received TERM signal\n");
    fprintf(stderr, "IPTraf terminated\n\n");
    exit(0);
}

/* 
 * Handler for the SIGSEGV, Segmentation Fault.  Tries to clear the screen
 * and issue a better message than "Segmentation fault".
 */

void segvhandler()
{
    erase();
    refresh();
    endwin();
    unlink(IPTIDFILE);
    fprintf(stderr, "Fatal error: unable to allocate memory for a critical function\n");
    fprintf(stderr, "IPTraf terminated abnormally\n\n");
    exit(2);
}

/*
 * Command-line help facility.
 */

void commandhelp()
{
    printf("\nCommand syntax: iptraf [ { -i | -g | -d iface | -s iface | -e | -f } ]\n\n");
    printf("Issuing the iptraf command without command-line options brings the program\n");
    printf("up in interactive mode, with the various facilities accessed through a menu.\n\n");
    printf("These options can also be supplied to the command:\n\n");
    printf("	-i		- immediately start the IP traffic monitor\n");
    printf("	-g		- immediately start the general interface statistics\n");
    printf("	-d iface	- allows you to immediately start the detailed\n");
    printf("			  on the indicated interface (iface)\n");
    printf("	-s iface	- allows you to immediately monitor TCP and UDP\n");
    printf("			  traffic on the specified interface (iface)\n");
    printf("	-e		- start the Ethernet station monitor\n");
    printf("	-f		- IPTraf normally will not allow multiple copies of\n");
    printf("			  itself running simultaneously.  -f overrides.\n");
    printf("			  Use with great caution.  May be used after an\n");
    printf("			  abnormal termination.\n\n");
    printf("IPTraf %s Copyright (c) Gerard Paul Java 1997, 1998\n", VERSION);
    printf("This software may be distributed and/or modified under the terms of the\n");
    printf("GNU General Public License, Version 2 or later as published by the Free\n");
    printf("Software Foundation.\n\n");
}

int first_instance()
{
    int fd;
    
    fd = open(IPTIDFILE, O_RDONLY);
    
    if (fd < 0)
        return 1;
    else {
        close(fd);
        return 0;
    }
}

void mark_first_instance()
{
   int fd;
   
   fd = open(IPTIDFILE, O_CREAT | O_WRONLY | O_TRUNC, S_IRUSR | S_IWUSR);
   if (fd < 0) {
       fprintf(stderr, "\nWarning: unable to tag this process\n");
       fprintf(stderr, "Press Enter to continue\n");
       getchar();
       return;
   }
   
   close(fd);
}

/*
 * Main routine
 */
 
int main(int argc, char **argv)
{
    struct OPTIONS options;
    extern char *optarg;
    extern int optind;
    extern int opterr;
    extern int optopt;
    int opt = 0;
    int command = 0;
    int keyargs = argc;
    char keyparm[12];

#ifndef ALLOWUSERS
    if (getuid() != 0) {
	fprintf(stderr, "\nIPTraf Version %s\n", VERSION);
	fprintf(stderr, "Copyright (c) Gerard Paul Java 1997, 1998\n\n");
	fprintf(stderr, "This program can be run only by the system administrator\n\n");
	exit(1);
    }
#endif

    /*
     * Parse command line
     */
     
    if (argc > 1) {
        do {
   	    opterr = 0;
	    opt = getopt(argc, argv, "igd:s:ehf");

	    if (opt == 'h') {
	        commandhelp();
	        exit(0);
	    } else if (opt == 'f') {
	        unlink(IPTIDFILE);
	        keyargs--;
	    } else if (opt == '?') {
	        fprintf(stderr, "\nInvalid option or missing parameter\n\n");
	        exit(1);
	    } else if (opt != -1) {
	        if (optarg != 0) {
  	            bzero(keyparm, 12);
	            strncpy(keyparm, optarg, 11);
	        }
	        command = opt;
	    }
	} while ((opt != '?') && (opt != -1));
    }

    if (!first_instance()) {
        fprintf(stderr, "\nOnly one copy of IPTraf can run at any one time\n");
        fprintf(stderr, "You can override this restriction with the -f option\n");
        fprintf(stderr, "Use -f with caution\n\n");
        exit(3);
    }
    
    
    signal(SIGTERM, (void *) termhandler);
    signal(SIGSEGV, (void *) segvhandler);
    signal(SIGTSTP, SIG_IGN);
    signal(SIGINT, SIG_IGN);

    if (getenv("TERM") == NULL) {
	fprintf(stderr, "Your TERM variable is not set.\n");
	fprintf(stderr, "Please set it to an appropriate value.\n");
	exit(1);
    }
    loadoptions(&options);

    initscr();

    if ((LINES < 25) || (COLS < 80)) {
	endwin();
	fprintf(stderr, "\nThis program requires a screen size of at least 80 columns by 25 lines\n");
	fprintf(stderr, "Please resize your window\n\n");
	exit(1);
    }

    mark_first_instance();
    
    start_color();
    standardcolors(options.color);
    noecho();
    nonl();
    cbreak();
    curs_set(0);

    srpromisc(options.promisc);
    
    program_interface(&options, keyargs, command, keyparm);

    srpromisc(0);

    endwin();

    unlink(IPTIDFILE);
        
    return (0);
}
